/*  Copyright 2010-2011 Guillaume Duhamel

    This file is part of Gophrier.

    Gophrier is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Gophrier is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gophrier; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
*/

#include <sys/socket.h>
#include <stdio.h>

#include "client.h"
#include "pool.h"
#include "request.h"
#include "log.h"

static char * client_socket_read(Socket * sock);

Socket * client_socket_new(int fd)
{
    Socket * sock;

    sock = socket_new(fd);
    sock->onRead = client_socket_on_read;

    return sock;
}

void client_socket_on_read(Socket * sock, SocketPool * pool)
{
    char * path = client_socket_read(sock);

    if (path != NULL)
    {
        Request * request;

        log_append(path);
        
        request = request_new(sock, path);
        free(path);
        request_exec(request);
        request_free(request);
    }

    socket_pool_remove(pool, sock);
}

char * client_socket_read(Socket * sock)
{
    char buffer[1024];
    char * path = NULL;
    unsigned int pathsize = 0;
    ssize_t got;
    int i = 0, j;
    int requestok = 0;

    while(! requestok)
    {
        got = recv(sock->fd, buffer, 1024, MSG_DONTWAIT);
        switch(got)
        {
            case -1:
                break;
            case 0:
                requestok = 1;
                break;
            default:
                pathsize += got;
                path = realloc(path, pathsize);
                j = 0;
                while((j < got) && (buffer[j] != '\r'))
                {
                    path[i] = buffer[j];
                    i++; j++;
                }
                
                if (buffer[j] == '\r')
                    requestok = 1;
                break;
        }
    }

    if (path != NULL)
        path[i] = 0;

    return path;
}
